"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.has = exports.addSingletone = exports.get = exports.addDependency = exports.register = void 0;
const container_1 = require("./container");
const registry = new Map();
const instances = new Map();
function register(constructor) {
    if (!registry.has(constructor)) {
        registry.set(constructor, []);
    }
}
exports.register = register;
function addDependency(target, dep, index) {
    const deps = registry.get(target) || [];
    const d = deps.find(f => f.constructor === dep);
    if (d) {
        d.index = index;
    }
    else {
        deps.push({ constructor: dep, index });
    }
    registry.set(target, deps);
}
exports.addDependency = addDependency;
function get(token) {
    if (instances.has(token)) {
        return instances.get(token);
    }
    const ctor = getConstructor(token);
    const dependencies = registry.get(ctor);
    if (!dependencies) {
        throw new Error(`Dependency injector: Type [${typeof token === 'function' ? token.name : token}] is not registered`);
    }
    let instance = null;
    if (!dependencies.length) {
        instance = new ctor();
        instances.set(token, instance);
    }
    else {
        const args = Array.from(dependencies).sort((left, right) => left.index - right.index).map(m => get(m.constructor));
        instance = new ctor(...args);
    }
    return instance;
}
exports.get = get;
function getConstructor(token) {
    return typeof token === 'function' ? token : container_1.resolveType(token);
}
function addSingletone(type, implementation) {
    container_1.bind(type, implementation);
    register(implementation);
}
exports.addSingletone = addSingletone;
function has(t) {
    return registry.has(t);
}
exports.has = has;
//# sourceMappingURL=registry.js.map